#!/bin/sh

#
# VARIABLES
#

# Determine if this is a cross environment or MSYS on Windows
if [ "$OSTYPE" == "msys" ] ; then
	# MSYS on Windows
	MINGWROOT="/mingw"
	TOOLPREFIX=""
	MAKEVARS="WXINSTALL=/usr/local"
else
	# Attempt to determine MinGW's cross-compile ID
	if [ -d /usr/i586-pc-mingw32 ] ; then
		MINGW=i586-pc-mingw32
	elif [ -d /usr/i586-mingw32msvc ] ; then
		MINGW=i586-mingw32msvc
	fi
	
	MINGWROOT="/usr/$MINGW"
	TOOLPREFIX="$MINGW-"
	MAKEVARS="TOOLPREFIX=$TOOLPREFIX LIBSUFFIX=-$MINGW WXINSTALL=$MINGWROOT"
fi

MAKE="make PLATFORM=WIN32 $MAKEVARS"


#
# FUNCTIONS
#

# Make the Windows installer using Inno Setup
make_windows_installer()
{
	read -p "Please enter the version: " VERSION
	read -p "Please enter the patch level: " PATCH
	read -p "Please enter the type (e.g. snapshot, release): " TYPE
	read -p "Please enter the distribution (e.g. unicode, ansi): " DIST
	
	# Get the Bazaar revision number if available

	echo "Getting Bazaar revision number..."
	if which bzr > /dev/null
	then
		BAZAARREV=`bzr revno`
		echo "    Revision number is $BAZAARREV"
		echo ""
	else
		BAZAARREV="user"
		echo "    Revision number not found or Bazaar not installed, using \"user\""
		echo ""
	fi

	# Show the user the filename being built

	echo "Building rocrail-setup-$VERSION.$PATCH-rev$BAZAARREV-$TYPE-$DIST.exe"
	echo ""

	# Strip binaries

	echo "Stripping Windows Binaries"
	${TOOLPREFIX}strip winbin/*.dll
	${TOOLPREFIX}strip winbin/*.exe

	echo "    Done"
	echo ""

	# Extract mingw runtime dll to winbin directory

	echo "Copying mingwm10.dll"

	if [ -f /usr/share/doc/mingw32-runtime/mingwm10.dll.gz ] ;	then
	  echo "    Using /usr/share/doc/mingw32-runtime/mingwm10.dll.gz"

	  cp /usr/share/doc/mingw32-runtime/mingwm10.dll.gz winbin
	  gunzip winbin/mingwm10.dll.gz

	elif [ -f /usr/share/doc/mingw32-runtime/mingwm10.dll ] ; then
	  echo "    Copying existing /usr/share/doc/mingw32-runtime/mingwm10.dll"

	  cp /usr/share/doc/mingw32-runtime/mingwm10.dll winbin

	elif [ -f $MINGWROOT/bin/mingwm10.dll ] ; then
	  echo "    Copying existing MSYS MinGW /mingw/bin/mingwm10.dll"

	  cp $MINGWROOT/bin/mingwm10.dll winbin
	else
	  echo "Error: mingwm10.dll not found!"
	  echo ""
	  echo "    Cannot find mingwm10.dll.gz or mingwm10.dll in "
	  echo "       /usr/share/doc/mingw32-runtime or $MINGWROOT/bin."
	  echo "    Please reinstall MinGW32 and make sure this file exists."
	  echo ""
	  exit $?
	fi

	echo "    Done"
	echo ""

	# How to automate the Windows Installer run?
	# How to pass in rev and other version numbers?
	#   Sed the file
	#     replace "revxxxx" with rev<bzr #>
	#     replace "1.2.0" with snapshot number  

	# Substitute variables in the default rocrail.iss file

	echo "Creating Inno Setup installer file..."

	# Run sed to swap version, bazaar rev number, release type and distribution

	sed -e "s:<BZR>:$BAZAARREV:" \
		-e "s:<VER>:$VERSION:" \
		-e "s:<PATCH>:$PATCH:" \
		-e "s:<DIST>:$DIST:" \
		-e "s:<TYPE>:$TYPE:" \
		< rocrail/package/rocrail-template.iss > rocrail/package/rocrail-temp.iss

	echo "    Done"
	echo ""

	# Run Inno Setup

	echo "Running Inno Setup..."
	echo ""

	if [ "$OSTYPE" == "msys" ] ; then
		/c/Program\ Files/Inno\ Setup\ 5/ISCC.exe rocrail/package/rocrail-temp.iss
	else
		wine "c:\\Program Files\\Inno Setup 5\\ISCC.exe" rocrail/package/rocrail-temp.iss
	fi
	rm rocrail/package/rocrail-temp.iss

	echo "    Done"
	echo ""

	# Show finished script

	echo "    Done building windows package"
	echo ""
}


#
# MAIN PROGRAM
#

# Determine if a make target was specified
if [ -z "$1" ] ; then
	$MAKE || exit $?
else
	# Process the make targets
	for arg in $* ; do
		if [ "$arg" == "install" ] ; then
			make_windows_installer
		elif [ "$arg" == "uninstall" ] ; then
			echo "    Uninstall on Windows is done through Add or Remove Programs"
			echo ""
			exit 1
		else
			$MAKE $arg || exit $?
		fi
	done
fi

exit $?
